import { __decorate } from "tslib";
import { designTokens as alphaDefaultDesignTokens } from '@genesislcap/alpha-design-system';
import { loadRobotoFontFaces, designTokens as zeroDefaultDesignTokens, robotoFontFamily, } from '@genesislcap/foundation-zero';
import { StandardLuminance, Combobox, Select } from '@microsoft/fast-components';
import { FASTElement, attr, customElement, observable, DOM } from '@microsoft/fast-element';
import { RadioGroup, Slider } from '@microsoft/fast-foundation';
import { DesignSystemEditorStyles as styles } from './design-system-editor.styles';
import { DesignSystemEditorTemplate as template } from './design-system-editor.template';
const fontFamilies = {
    Roboto: robotoFontFamily,
    'Segoe UI': '"Segoe UI", Arial, Helvetica, sans-serif',
    Monospace: 'Consolas, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", Monaco, "Courier New", Courier, monospace',
    'Times New Roman': 'Cambria, "Hoefler Text", Utopia, "Liberation Serif", "Nimbus Roman No9 L Regular", Times, "Times New Roman", serif',
    Garamond: '"Palatino Linotype", Palatino, Palladio, "URW Palladio L", "Book Antiqua", Baskerville, "Bookman Old Style", "Bitstream Charter", "Nimbus Roman No9 L", Garamond, "Apple Garamond", "ITC Garamond Narrow", "New Century Schoolbook", "Century Schoolbook", "Century Schoolbook L", Georgia, serif',
    Georgia: 'Constantia, "Lucida Bright", Lucidabright, "Lucida Serif", Lucida, "DejaVu Serif", "Bitstream Vera Serif", "Liberation Serif", Georgia, serif',
    Helvetica: 'Frutiger, "Frutiger Linotype", Univers, Calibri, "Gill Sans", "Gill Sans MT", "Myriad Pro", Myriad, "DejaVu Sans Condensed", "Liberation Sans", "Nimbus Sans L", Tahoma, Geneva, "Helvetica Neue", Helvetica, Arial, sans-serif',
    Verdana: 'Candara, "Bitstream Vera Sans", "DejaVu Sans", "Bitstream Vera Sans", "Trebuchet MS", Verdana, "Verdana Ref", sans-serif',
};
const mapDesignToken = ({ design_tokens: dsValues, }) => {
    return {
        accentColor: dsValues.color.accent.$value,
        neutralColor: dsValues.color.neutral.$value,
        luminance: dsValues.mode.luminance.$value,
        density: dsValues.style.density.$value,
        designUnit: dsValues.space.designUnit.$value,
        borderRadius: dsValues.style.borderRadius.$value,
        strokeWidth: dsValues.style.strokeWidth.$value,
        bodyFont: Object.keys(fontFamilies).find((font) => fontFamilies[font] === dsValues.fontFamily.bodyFont.$value),
        baseFontSize: dsValues.typography.baseFontSize.$value,
        baseLineHeight: dsValues.typography.baseLineHeight.$value,
    };
};
const ZeroDefaults = mapDesignToken(zeroDefaultDesignTokens);
const ZeroButtonApperance = {
    accent: 'primary-blue',
    neutral: 'neutral-grey',
    outline: 'primary-gradient',
};
const AlphaDefaults = mapDesignToken(alphaDefaultDesignTokens);
const AlphaButtonApperance = {
    accent: 'accent',
    neutral: 'neutral',
    outline: 'outline',
};
const DS = {
    zero: {
        tokens: ZeroDefaults,
        buttons: ZeroButtonApperance,
    },
    alpha: {
        tokens: AlphaDefaults,
        buttons: AlphaButtonApperance,
    },
};
export var EventName;
(function (EventName) {
    EventName["STYLES_CHANGED"] = "styles-changed";
    EventName["DS_CHANGED"] = "design-system-changed";
})(EventName || (EventName = {}));
let DesignSystemEditor = class DesignSystemEditor extends FASTElement {
    constructor() {
        super(...arguments);
        this.customDesignMode = false;
        this.ds = 'zero';
        this.darkMode = ZeroDefaults.luminance === StandardLuminance.DarkMode;
        this.luminance = ZeroDefaults.luminance;
        this.neutralColor = ZeroDefaults.neutralColor;
        this.accentColor = ZeroDefaults.accentColor;
        this.bodyFont = ZeroDefaults.bodyFont;
        this.density = ZeroDefaults.density;
        this.borderRadius = ZeroDefaults.borderRadius;
        this.strokeWidth = ZeroDefaults.strokeWidth;
        this.designUnit = ZeroDefaults.designUnit;
        this.baseFontSize = ZeroDefaults.baseFontSize;
        this.baseLineHeight = ZeroDefaults.baseLineHeight;
        this.changeHandler = (e) => {
            if (e.target instanceof RadioGroup ||
                e.target instanceof HTMLInputElement ||
                e.target instanceof Combobox ||
                e.target instanceof Select) {
                const { name, value } = e.target;
                if (name && value) {
                    this[name] = value;
                    this.styleUpdate();
                }
            }
            else if (e.target instanceof Slider) {
                const value = parseInt(e.target.value);
                const name = e.target.name;
                if (name && Number.isInteger(value)) {
                    this[name] = value;
                    this.styleUpdate();
                }
            }
        };
        this.modeChangeHandler = (e) => {
            this.darkMode = e.target.checked;
            this.luminance = this.darkMode ? StandardLuminance.DarkMode : StandardLuminance.LightMode;
            this.styleUpdate();
        };
        this.styleUpdate = () => {
            const detail = {
                accentColor: this.accentColor,
                neutralColor: this.neutralColor,
                bodyFont: fontFamilies[this.bodyFont],
                luminance: this.luminance,
                density: this.density,
                designUnit: this.designUnit,
                borderRadius: this.borderRadius,
                strokeWidth: this.strokeWidth,
                baseFontSize: this.baseFontSize,
                baseLineHeight: this.baseLineHeight,
            };
            this.$emit(EventName.STYLES_CHANGED, detail);
        };
        this.setTokensFromDs = (ds) => {
            this.neutralColor = ds.neutralColor;
            this.accentColor = ds.accentColor;
            this.bodyFont = ds.bodyFont;
            this.density = ds.density;
            this.designUnit = ds.designUnit;
            this.borderRadius = ds.borderRadius;
            this.strokeWidth = ds.strokeWidth;
            this.darkMode = ds.luminance === StandardLuminance.DarkMode;
            this.baseFontSize = ds.baseFontSize;
            this.baseLineHeight = ds.baseLineHeight;
        };
        this.resetChanges = () => {
            this.setTokensFromDs(DS[this.ds].tokens);
            this.styleUpdate();
        };
    }
    customDesignModeChanged(oldValue, newValue) {
        const defaultDesignSystems = [
            { name: 'Zero', value: 'zero' },
            { name: 'Foundation UI', value: 'alpha' },
        ];
        const customDesignSystem = { name: 'Custom Design', value: 'zero' };
        this.designSystems = newValue
            ? [customDesignSystem, ...defaultDesignSystems]
            : defaultDesignSystems;
    }
    dsChanged(oldValue, newValue) {
        const detail = {
            prefix: this.ds,
            buttonAppearanceAccent: DS[this.ds].buttons.accent,
            buttonAppearanceNeutral: DS[this.ds].buttons.neutral,
            buttonAppearanceOutline: DS[this.ds].buttons.outline,
        };
        this.$emit(EventName.DS_CHANGED, detail);
    }
    connectedCallback() {
        super.connectedCallback();
        loadRobotoFontFaces();
        DOM.queueUpdate(() => {
            this.styleUpdate();
        });
    }
};
__decorate([
    attr({ mode: 'boolean', attribute: 'custom-design-mode' })
], DesignSystemEditor.prototype, "customDesignMode", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "designSystems", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "ds", void 0);
__decorate([
    attr
], DesignSystemEditor.prototype, "darkMode", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "neutralColor", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "accentColor", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "bodyFont", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "density", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "borderRadius", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "strokeWidth", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "designUnit", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "baseFontSize", void 0);
__decorate([
    observable
], DesignSystemEditor.prototype, "baseLineHeight", void 0);
DesignSystemEditor = __decorate([
    customElement({
        name: 'design-system-editor',
        template,
        styles,
    })
], DesignSystemEditor);
export { DesignSystemEditor };
